@@ -12,21 +12,25 @@ import os
 import pathlib
 import re
 
+from typing import Any, Dict, Optional, Union
+
 import yaml
 
 
-__all__ = ['read_yaml_file', 'merge_config', 'get_config', 'Configuration']
+__all__ = ["read_yaml_file", "merge_config", "get_config", "Configuration"]
 
 
 __ENVVARS__ = {}
 
 # Potential paths where the user configuration file could be. Must not include
 # the yml or yaml extension.
-DEFAULT_PATHS = ['~/.config/sdss/{name}',
-                 '~/.config/sdss/{name}/{name}',
-                 '~/.{name}/{name}']
+DEFAULT_PATHS = [
+    "~/.config/sdss/{name}",
+    "~/.config/sdss/{name}/{name}",
+    "~/.{name}/{name}",
+]
 
-env_matcher = re.compile(r'\$\{([^}^{]+)\}')
+env_matcher = re.compile(r"\$\{([^}^{]+)\}")
 
 
 def env_constructor(loader, node):
@@ -36,23 +40,33 @@ def env_constructor(loader, node):
     match = env_matcher.match(value)
     env_var = match.group()[2:-1]
 
-    return os.environ.get(env_var, __ENVVARS__.get(env_var, value)) + value[match.end():]
+    def_env_var = __ENVVARS__.get(env_var, value)
+
+    return os.environ.get(env_var, def_env_var) + value[match.end() :]
 
 
-yaml.add_implicit_resolver('!env', env_matcher)
-yaml.add_constructor('!env', env_constructor)
+yaml.add_implicit_resolver("!env", env_matcher)
+yaml.add_constructor("!env", env_constructor)
 
 
-def read_yaml_file(path, use_extends=True, loader=yaml.FullLoader):
+ConfigType = Dict[str, Any]
+AnyPath = Union[str, pathlib.Path]
+
+
+def read_yaml_file(
+    path: AnyPath,
+    use_extends: bool = True,
+    loader: Any = yaml.FullLoader,
+) -> ConfigType:
     """Read a YAML file and returns a dictionary."""
 
     if isinstance(path, (str, pathlib.Path)):
-        fp = open(path, 'r')
+        fp = open(path, "r")
     else:
         fp = path
 
     fp.seek(0)
-    config = yaml.load(fp, Loader=loader)
+    config: Union[ConfigType, None] = yaml.load(fp, Loader=loader)
 
     if config is None or config == {}:
         return {}
@@ -60,21 +74,22 @@ def read_yaml_file(path, use_extends=True, loader=yaml.FullLoader):
     if use_extends:
         fp.seek(0)
         for line in fp.readlines():
-            if line.strip().startswith('#!extends'):
+            if line.strip().startswith("#!extends"):
                 base_file = line.strip().split()[1]
-                if not os.path.isabs(base_file) and hasattr(fp, 'buffer'):
+                if not os.path.isabs(base_file) and hasattr(fp, "buffer"):
                     base_file = os.path.join(os.path.dirname(path), base_file)
                 if not os.path.exists(base_file):
-                    raise FileExistsError(f'cannot find !extends file {base_file}.')
-                return merge_config(read_yaml_file(base_file, use_extends=False),
-                                    config)
-            elif line.strip().startswith('#') or line.strip() == '':
+                    raise FileExistsError(f"cannot find !extends file {base_file}.")
+                return merge_config(
+                    read_yaml_file(base_file, use_extends=False), config
+                )
+            elif line.strip().startswith("#") or line.strip() == "":
                 continue
 
     return config
 
 
-def merge_config(user, default):
+def merge_config(user: ConfigType, default: ConfigType) -> ConfigType:
     """Merges a user configuration with the default one."""
 
     if isinstance(user, dict) and isinstance(default, dict):
@@ -87,8 +102,15 @@ def merge_config(user, default):
     return user
 
 
-def get_config(name, config_file=None, allow_user=True, user_path=None,
-               config_envvar=None, merge_mode='update', default_envvars={}):
+def get_config(
+    name: str,
+    config_file: Optional[AnyPath] = None,
+    allow_user: bool = True,
+    user_path: Optional[AnyPath] = None,
+    config_envvar: Optional[str] = None,
+    merge_mode: str = "update",
+    default_envvars: Dict[str, Any] = {},
+) -> "Configuration":
     """Returns a configuration dictionary.
 
     The configuration dictionary is created by merging the default
@@ -100,62 +122,60 @@ def get_config(name, config_file=None, allow_user=True, user_path=None,
 
     Parameters
     ----------
-    name : str
+    name
         The name of the package.
-    config_file : str
+    config_file
         The path to the configuration file. If `None`, defaults to
         ``etc/<name>.yml`` relative to the file that called `.get_config`.
-    allow_user : bool
+    allow_user
         If `True`, looks for an user configuration file and merges is to the
         default configuration. Otherwise it returns just the default
         configuration.
-    user_path : str
+    user_path
         The path to the user configuration file. Defaults to
         ``~/.config/sdss/<name>/<name>.yml``. Ignored if the file does not
         exist.
-    config_envvar : str
+    config_envvar
         The environment variable that contains the path to the user
         configuration file. Defaults to ``<name>_CONFIG_PATH``. If the
         environment variable exists, the ``user_path`` is ignored.
-    merge_mode : str
+    merge_mode
         Defines how the default and user dictionaries will be merged. If
         ``update``, the user dictionary will be used to update the default
         configuration. If ``replace``, only the user configuration will be
         returned.
-    default_envvars : dict
+    default_envvars
         Default values for environment variables used in the configuration
         file.
 
     Returns
     -------
-    .Configuration
+    Configuration
         A `.Configuration` instance.
-
     """
 
-    assert merge_mode in ['update', 'replace'], 'invalid merge mode.'
+    assert merge_mode in ["update", "replace"], "invalid merge mode."
 
     if not config_file:
         try:
             frame = inspect.stack()[1]
             module = inspect.getmodule(frame[0])
             dirname = os.path.dirname(module.__file__)
-            config_file = os.path.join(dirname, f'etc/{name}.yml')
+            config_file = os.path.join(dirname, f"etc/{name}.yml")
         except AttributeError:
             config_file = None
 
     if allow_user is False:
-        return Configuration(base_config=config_file,
-                             default_envvars=default_envvars)
+        return Configuration(base_config=config_file, default_envvars=default_envvars)
 
-    config_envvar = config_envvar or '{}_CONFIG_PATH'.format(name.upper())
+    config_envvar = config_envvar or "{}_CONFIG_PATH".format(name.upper())
 
     if user_path is not None:
         user_path = os.path.expanduser(os.path.expandvars(user_path))
-        assert os.path.exists(user_path), f'User path {user_path!r} not found.'
+        assert os.path.exists(user_path), f"User path {user_path!r} not found."
     else:
         # Test a few default paths and exit when finds one.
-        extensions = ['.yaml', '.yml']
+        extensions = [".yaml", ".yml"]
         for path, extension in itertools.product(DEFAULT_PATHS, extensions):
             path = str(path).format(name=name)
             test_path = os.path.expanduser(path + extension)
@@ -170,12 +190,14 @@ def get_config(name, config_file=None, allow_user=True, user_path=None,
     else:
         custom_config_fn = None
 
-    if merge_mode == 'update':
-        return Configuration(custom_config_fn, base_config=config_file,
-                             default_envvars=default_envvars)
+    if merge_mode == "update":
+        return Configuration(
+            custom_config_fn, base_config=config_file, default_envvars=default_envvars
+        )
     else:
-        return Configuration(custom_config_fn, base_config=None,
-                             default_envvars=default_envvars)
+        return Configuration(
+            custom_config_fn, base_config=None, default_envvars=default_envvars
+        )
 
 
 class Configuration(dict):
@@ -183,18 +205,22 @@ class Configuration(dict):
 
     Parameters
     ----------
-    config : str or dict
+    config
         The path to the configuration file or the already parsed configuration
         as a dictionary.
-    base_config : str or dict
+    base_config
         A base configuration or file that the input configuration will update.
-    default_envvars : dict
+    default_envvars
         Default values for environment variables used in the configuration
         file.
-
     """
 
-    def __init__(self, config=None, base_config=None, default_envvars={}):
+    def __init__(
+        self,
+        config: Optional[Union[AnyPath, ConfigType]] = None,
+        base_config: Optional[Union[AnyPath, ConfigType]] = None,
+        default_envvars: Dict[str, Any] = {},
+    ):
 
         global __ENVVARS__
 
@@ -221,21 +247,20 @@ class Configuration(dict):
             elif isinstance(config, (str, pathlib.Path)):
                 return read_yaml_file(config)
             else:
-                raise ValueError('Invalid config of type {}'.format(type(config)))
+                raise ValueError("Invalid config of type {}".format(type(config)))
 
         return merge_config(self._parse_config(config, use_base=False), self._BASE)
 
-    def load(self, config=None):
+    def load(self, config: Optional[Union[AnyPath, ConfigType]] = None):
         """Loads a configuration file.
 
         Parameters
         ----------
-        config : str or dict
+        config
             The configuration file or dictionary to load. If a base
             configuration was defined when the object was instantiated, it
             will be merged. If ``config=None``, the object will revert to the
             base configuration.
-
         """
 
         if config is None:
